@@ -2,5 +2,5 @@ from .points import Dot, CloudPoints
 from .variograms  import Variogram, Spherical, Exponential, Gaussian
 from .transforms import to_normal, NScaler
 from .grid import Grid
-from .krigging import OrdinaryKrigging, IndicatorOridinaryKrigging, ordinary_krigging
+from .krigging import OrdinaryKrigging, IndicatorOridinaryKrigging, ordinary_krigging, IWD, inverse_weighted_distance
 from .sgs import SequentialGaussianSimulation
\ No newline at end of file
